# GromacsWrapper: test_example.py
# Copyright (c) 2009 Oliver Beckstein <orbeckst@gmail.com>
# Released under the GNU Public License 3 (or higher, your choice)
# See the file COPYING for details.

import gromacs
import pytest
from numpy.testing import assert_equal

from ..datafiles import datafile

@pytest.fixture(
    params=['original', 'written', 'no_autoconvert'],
)
def CUSTOM_EM_MDP(request, tmpdir):
    autoconvert = not request.param == 'no_autoconvert'
    mdp = gromacs.fileformats.mdp.MDP(datafile('custom_em.mdp'),
                                      autoconvert=autoconvert)
    if request.param == 'written':
        # to check that written mdp has same data
        out = str(tmpdir.join('out.mdp'))
        mdp.write(out)
        mdp = gromacs.fileformats.mdp.MDP(out)
    return mdp, autoconvert


class TestMDP(object):
    def test_values(self, CUSTOM_EM_MDP):
        mdp, autoconvert = CUSTOM_EM_MDP

        if autoconvert:
            def conv(val):
                return val
        else:
            def conv(val):
                return str(val)

        if autoconvert:
            assert_equal(mdp['include'], ['-I.', '-I..', '-I../top'])
            assert_equal(mdp['define'], '-DPOSRES')
        else:
            assert mdp['include'] == '-I. -I.. -I../top'
            assert mdp['define'] == '-DPOSRES'
        assert mdp['integrator'] == 'cg'
        assert mdp['emtol'] == conv(500)
        assert mdp['emstep'] == conv(0.01)
        assert mdp['nsteps'] == conv(1000)
        assert mdp['nstcgsteep'] == conv(100)
        assert mdp['constraints'] == 'none'
        assert mdp['nstcomm'] == conv(1)
        assert mdp['cutoff-scheme'] == 'Verlet'
        assert mdp['vdwtype'] == 'cutoff'
        assert mdp['coulombtype'] == 'PME'
        assert mdp['ns_type'] == 'grid'
        assert mdp['rlist'] == conv(1.0)
        assert mdp['rcoulomb'] == conv(1.0)
        assert mdp['rvdw'] == conv(1.0)
        assert mdp['rvdw-switch'] == conv(0.8)
        assert mdp['Tcoupl'] == 'no'
        assert mdp['Pcoupl'] == 'no'
        assert mdp['gen_vel'] == 'no'
        assert mdp['nstxout'] == conv(0)

    def test_comments(self, CUSTOM_EM_MDP):
        mdp, _ = CUSTOM_EM_MDP

        assert mdp['C0001'] == 'custom EM'
        assert sum(1 for k in mdp if k.startswith('C0')) == 1

    def test_blank_lines(self, CUSTOM_EM_MDP):
        mdp, _ = CUSTOM_EM_MDP
        assert sum(1 for k in mdp if k.startswith('B0')) == 6

    def test_no_filename(self, tmpdir):
        mdp = gromacs.fileformats.mdp.MDP()

        mdp['define'] = ['-DPOSRES', '-DTHIS']
        mdp['vdwtype'] = 'cutoff'
        mdp['nsteps'] = 1234

        out = str(tmpdir.join('out.mdp'))
        mdp.write(out)

        back = gromacs.fileformats.mdp.MDP(out)

        assert_equal(back['define'], ['-DPOSRES', '-DTHIS'])
        assert back['vdwtype'] == 'cutoff'
        assert back['nsteps'] == 1234

@pytest.fixture
def NONSENSE_MDP(tmpdir):
    outfile = str(tmpdir.join('nonsense.mdp'))

    with open(datafile('custom_em.mdp'), 'r') as infile:
        data = infile.read()

    data += 'errors: plenty\n'

    with open(outfile, 'w') as out:
        out.write(data)

    return outfile


def test_bad_mdp(NONSENSE_MDP):
    with pytest.raises(gromacs.ParseError,
                       match="unknown line in mdp file, 'errors: plenty'"):
        gromacs.fileformats.mdp.MDP(NONSENSE_MDP)



# GromacsWrapper: test_amber03star.py
# Copyright (c) 2009 Oliver Beckstein <orbeckst@gmail.com>
# Released under the GNU Public License 3 (or higher, your choice)
# See the file COPYING for details.

from __future__ import division, absolute_import, print_function

import pytest

import gromacs

from .top import TopologyTest
from ...datafiles import datafile

@pytest.mark.xfail(gromacs.release().startswith("2022"),
                   reason="issue https://github.com/Becksteinlab/GromacsWrapper/issues/236")
class TestAmber03star(TopologyTest):
        processed = datafile('fileformats/top/amber03star/processed.top')
        conf = datafile('fileformats/top/amber03star/conf.gro')
        molecules = ['Protein', 'SOL', 'IB+', 'CA', 'CL', 'NA', 'MG', 'K', 'RB', 'CS', 'LI', 'ZN']



# -*- coding: utf-8 -*-
# GromacsWrapper
# Copyright (c) 2009 Oliver Beckstein <orbeckst@gmail.com>
# Released under the GNU Public License 3 (or higher, your choice)
# See the file COPYING for details.

from __future__ import division, absolute_import, print_function

import pytest
from numpy.testing import assert_equal, assert_almost_equal

from gromacs.fileformats import convert


@pytest.mark.parametrize(
    's,expected',
    [(100, 100),
     ("Jabberwock", u"Jabberwock"),
     (u"Ångström", u"Ångström"),
    ]
)
def test_to_unicode(s, expected):
    output = convert.to_unicode(s)
    assert output == expected

class TestAutoconverter(object):
    def _convert(self, s, **kwargs):
        ac = convert.Autoconverter(**kwargs)
        assert ac.active is True
        return ac.convert(s)


    @pytest.mark.parametrize(
        "s,expected", [
            ('foo bar 22  boing ---', ('foo', 'bar', 22, 'boing', None)),
            ('1 2 3 4', (1, 2, 3, 4)),
            ('1    2  3 4', (1, 2, 3, 4)),
            ('True x X yes Present', (True, True, True, True, True)),
            ('False no - None none', (False, False, False, False, False))
        ],
    )
    @pytest.mark.parametrize('sep', (True, None))
    def test_convert_default(self, s, expected, sep):
        output = self._convert(s, sep=sep)
        assert_equal(output, expected)

    @pytest.mark.parametrize(
        "s,expected", [
            ('1,2,3,4', (1, 2, 3, 4)),
            ('1 2,3,4', ('1 2', 3, 4)),
        ]
    )
    def test_convert_default_sep(self, s, expected, sep=','):
        output = self._convert(s, sep=sep)
        assert_equal(output, expected)


    @pytest.mark.parametrize(
        "s,expected", [
            ('2.71213 3.14', (2.71213, 3.14)),
            ('1000 -234 987654', (1000, -234, 987654)),
        ]
    )
    @pytest.mark.parametrize('sep', (True, None))
    def test_convert_numbers(self, s, expected, sep):
        output = self._convert(s, sep=sep)
        assert_almost_equal(output, expected)





# setuptools installation of GromacsWrapper
# Copyright (c) 2008-2011 Oliver Beckstein <orbeckst@gmail.com>
# Released under the GNU Public License 3 (or higher, your choice)
#
# See the files INSTALL and README for details or visit
# https://github.com/Becksteinlab/GromacsWrapper
from __future__ import with_statement
from setuptools import setup, find_packages

import versioneer

with open("README.rst") as readme:
    long_description = readme.read()


setup(name="GromacsWrapper",
      version=versioneer.get_version(),
      cmdclass=versioneer.get_cmdclass(),
      description="A Python wrapper around the Gromacs tools.",
      long_description=long_description,
      long_description_content_type="text/x-rst",
      author="Oliver Beckstein",
      author_email="orbeckst@gmail.com",
      license="GPLv3",
      url="https://github.com/Becksteinlab/GromacsWrapper",
      download_url="https://github.com/Becksteinlab/GromacsWrapper/downloads",
      keywords="science Gromacs analysis 'molecular dynamics'",
      classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows ',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Chemistry',
        'Topic :: Software Development :: Libraries :: Python Modules',
      ],
      packages=find_packages(
          exclude=['scripts', 'tests', 'tests.*', 'extras', 'doc/examples']),
      scripts=[
          'scripts/gw-join_parts.py',
          'scripts/gw-merge_topologies.py',
          'scripts/gw-forcefield.py',
          'scripts/gw-partial_tempering.py',
      ],
      package_data={'gromacs': ['templates/*.sge', 'templates/*.pbs',  # template files
                                'templates/*.ll', 'templates/*.sh',
                                'templates/*.mdp', 'templates/*.cfg'
                                ],
                    },
      install_requires=['numpy>=1.0',
                        'six',          # towards py 3 compatibility
                        'numkit',       # numerical helpers
                        'matplotlib',
                        ],
      tests_require=['pytest', 'numpy>=1.0', 'pandas>=0.17'],
      zip_safe=True,
      )
